#!/usr/bin/ruby -w

#
# names_to_numbers.rb
#
# Renames all the files in a directory with the given file extension.
# The renamed files will just have numbers.
#
# Colin Mccabe
#

require 'fileutils'
require 'optparse'
require 'ostruct'

class MyOptions
  def self.parse(args)
    opts = OpenStruct.new
    opts.dry_run = false
    opts.num_digits = 2
    opts.extension = nil
    opts.starting_number = 1
    opts.prefix_counter_increment = 1
    $fu_args = { :verbose => true }
    opts.preserve_names = false

    # Fill in $opts values
    parser = OptionParser.new do |myparser|
      myparser.banner = "Usage: #{ File.basename($0) } [opts]"
      myparser.separator("Specific options:")
      myparser.on("--num-digits DIGITS", "-D",
              "Set the number of digits in the numbering scheme.") do |d|
        opts.num_digits = d.to_i
      end
      myparser.on("--dry-run", "-d",
              "Show what would be done, without doing it.") do |d|
        $fu_args = { :verbose => true, :noop => true }
        opts.dry_run = true
      end
      myparser.on("--file-extension EXTENSION", "-e",
              "The file extension for the files to rename.") do |e|
        opts.extension = e
      end
      myparser.on("--preserve-names", "-p",
              "Preserve the names while changing the numbers.") do |e|
        opts.preserve_names = true
      end
      myparser.on("--starting-number NUMBER", "-N",
              "The starting number (defaults to 1)") do |e|
        opts.starting_number = e.to_i
      end
      myparser.on("--glob GLOB", "-g",
              "Specify the glob expression to use.  Example: '*/*.mp3'.") do |d|
        opts.glob = d
      end
      myparser.on("--prefix-counter-increment NUMBER", "-I",
              "The increment to use (defaults to 1)") do |e|
        opts.prefix_counter_increment = e.to_i
      end
    end

    parser.parse!(args)
    raise "invalid num_digits: #{opts.num_digits}" unless
      opts.num_digits > 0
    raise "must specify an extension" unless opts.extension != nil
    if (opts.glob == nil) then
      opts.glob = "*.#{opts.extension}"
    end
    return opts
  end
end

def pow(x, y)
  ret = 1
  (0...y).each do |z|
    ret = ret * x
  end
  return ret
end
def file_iter
  Dir.glob($opts.glob).sort.each do |f|
    yield f
  end
end

def count_files(file)
  $total_files = $total_files + 1
end

def get_file_name(num)
  return sprintf("%0#{$opts.num_digits}d", num)
end

def rename_files(file)
  dst="#{get_file_name($prefix_counter)}.#{$opts.extension}"
  if (file != dst) then
    FileUtils.mv(file, dst, $fu_args)
  end
  $prefix_counter = $prefix_counter + $opts.prefix_counter_increment
end

def rename_files_keep_names(file)
  proper_file = ""
  if (file =~ /^[0-9. -]*(.*)$/) then
    proper_file = $1
  else
    raise "can't find proper name for #{file}"
  end
  full_name = "#{get_file_name($prefix_counter)} - #{proper_file}"
  FileUtils.mv(file, full_name, $fu_args)
  $prefix_counter = $prefix_counter + $opts.prefix_counter_increment
end

# MAIN
begin
  $opts = MyOptions.parse(ARGV)
rescue Exception => msg
  $stderr.print("#{msg}.\nType --help to see usage information.\n")
  exit 1
end

# make sure there aren't too many files
$total_files = 0
max_total_files = pow(10, $opts.num_digits) - 1
file_iter { |f| count_files(f) }
if ($total_files > max_total_files) then
  raise "With #{$opts.num_digits} digit(s), we can only have at most \
#{max_total_files} files-- but there are #{$total_files} files in the \
#directory. Try setting a higher value for num_digits, using -D."
end

# rename files
$prefix_counter = $opts.starting_number
if ( $opts.preserve_names ) then
  file_iter { |f| rename_files_keep_names(f) }
else
  file_iter { |f| rename_files(f) }
end

exit 0
